/*
    Rohan Murty	w/ Help from Todd Moore (for the snd card stuff)
	Streaming audio player using minithreads!
*/

#include <windows.h>
#include <math.h>
#include "defs.h"
#include "minithread.h"
#include "minimsg.h"
#include "synch.h"

#define BUFFERSIZE    4860                // 4k sound buffer

int startListening() {
HWAVEOUT     hWaveOut;          // Handle to sound card output
    WAVEFORMATEX WaveFormat;        // The sound format
    WAVEHDR      WaveHeader;        // WAVE header for our sound data
    
    char         Data[BUFFERSIZE];  // Sound data buffer
    
    HANDLE       Done;              // Event Handle that tells us the sound has finished being played.
                                    // This is a real efficient way to put the program to sleep
                                    // while the sound card is processing the sound buffer
    double x;
    int i;
	int length;
	miniport_t port;
	miniport_t from;

	port = miniport_local_create();
	
    // ** Initialize the sound format we will request from sound card **    
    WaveFormat.wFormatTag = WAVE_FORMAT_PCM;     // Uncompressed sound format
    WaveFormat.nChannels = 1;                    // 1=Mono 2=Stereo
    WaveFormat.wBitsPerSample = 8;               // Bits per sample per channel
    WaveFormat.nSamplesPerSec = 11025;           // Sample Per Second
    WaveFormat.nBlockAlign = WaveFormat.nChannels * WaveFormat.wBitsPerSample / 8;
    WaveFormat.nAvgBytesPerSec = WaveFormat.nSamplesPerSec * WaveFormat.nBlockAlign;    
    WaveFormat.cbSize = 0;

    // ** Create our "Sound is Done" event **
    Done = CreateEvent (0, FALSE, FALSE, 0);
    
    // ** Open the audio device **
    if (waveOutOpen(&hWaveOut,0,&WaveFormat,(DWORD) Done,0,CALLBACK_EVENT) != MMSYSERR_NOERROR) 
    {        
        printf("\nError opening sound card!");
		getch();
        exit(0);
    }

    /*Lets get the sound buffer*/    
	minimsg_receive(port, &from, Data, &length);
	printf("\nReceived packet of length %d\n",length);

    // ** Create the wave header for our sound buffer **
    WaveHeader.lpData=Data;
    WaveHeader.dwBufferLength=BUFFERSIZE;
    WaveHeader.dwFlags=0;
    WaveHeader.dwLoops=0;
    
    // ** Prepare the header for playback on sound card **
    if (waveOutPrepareHeader(hWaveOut,&WaveHeader,sizeof(WaveHeader)) != MMSYSERR_NOERROR)
    {
        printf("\nError! Preparing header!");
		getch();
        return TRUE;
    }
    
    // ** Play the sound! **
    ResetEvent(Done);  // Reset our Event so it is non-signaled
                       // When the buffer finishes, it will be signaled again

    if (waveOutWrite(hWaveOut,&WaveHeader,sizeof(WaveHeader)) != MMSYSERR_NOERROR)
    {
        printf("\nError writing to sound card!");
		getch();
        exit(0);
    }

    // ** Wait until sound finishes playing
    if (WaitForSingleObject(Done,INFINITE) == WAIT_OBJECT_0);

    
    // ** Unprepare our wav header **
    if (waveOutUnprepareHeader(hWaveOut,&WaveHeader,sizeof(WaveHeader)) != MMSYSERR_NOERROR)
    {
        printf("\nError in unpreparing the header!");
        return TRUE;
    }
    
    // ** close the wav device **
    if (waveOutClose(hWaveOut) != MMSYSERR_NOERROR)
    {
        printf("\nError! Soundcard cannot be closed!");
        return -1;
    } 

    return 0;
}

int main() 
{   
	printf("\nListening!");
	minithread_system_initialize(startListening,NULL);
}

